import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1

import "../databasemodule_v2.js" as DB
import "../."

Page {
    id: appendFeedPage

    objectName: "appendfeedpage"
    title: i18n.tr("Add feeds")
    flickable: null
    visible: false

    property bool isDirty: false // Need to clear all page content.
    property int selectedCount: 0
    //.property var selectedFeeds: null // Become local.
    property bool resultsReceived: false // Indicates that at least once results were received.

    // Workaround for bug #1231137
    // set the tools to null, then add the toolbar manually when enter this page

    tools: null

    ToolbarItems {
        id: appendFeedTools

        opened: true
        locked: true

        back: Button {
            anchors.verticalCenter: parent.verticalCenter
            action: Action {
                text:  i18n.tr("Cancel")

                onTriggered:
                {
                    googleFeedApi.abort()
                    pageStack.pop()
                }
            }
        }

        ActivityIndicator {
            id: checkRunning

            visible: googleFeedApi.inProgress
            running: googleFeedApi.inProgress

            anchors.verticalCenter: parent.verticalCenter
        }

        Button {
            objectName: "nextButton"
            id: nextBtn
            anchors.verticalCenter: parent.verticalCenter

            enabled: !googleFeedApi.inProgress && selectedCount > 0
            action: Action {
                text:  i18n.tr("Next")

                onTriggered:
                {
                    if (!nextBtn.enabled)
                        return

                    var selectedFeeds = []
                    for (var i = 0; i < searchResultsModel.count; i++) {
                        var curItem = searchResultsModel.get(i)

                        if (curItem.isSelected) {
                            selectedFeeds.push(curItem)
                        }
                    }

                    pageStack.push(chooseTopicPage, {"feedsToAdd" : selectedFeeds})
                }
            }
        }
    }

    function reloadPageContent() {
        appendFeedPage.tools = appendFeedTools
        if (isDirty) {
            tfFeedUrl.text = ""
            resultsReceived = false
            clearModelDependData()
        }
        isDirty = false
    }

    /* Clear model and model's depend data.
     * Currently only selectedCount.
     */
    function clearModelDependData() {
        searchResultsModel.clear()
        selectedCount = 0
    }

    // -------------------------------------       GOOGLE API

    Connections {
        target: googleFeedApi

        onFindResult: {
            resultsReceived = true

            if ( result.responseStatus !== 200 ) {
                // No need to show dialog in case of abort.
                if (result.responseStatus !== 0) {
                    PopupUtils.open(errorDialogComponent, appendFeedPage,
                                    {"text" : i18n.tr("Failed to perform a feed search by keyword"),
                                        "title" : i18n.tr("Search failed")})
                    console.log(JSON.stringify(result))
                }
                return
            }

            clearModelDependData()

            var entries = result.responseData.entries

            for (var i = 0; i < entries.length; i++) {
                searchResultsModel.append({"url" : entries[i].url,
                                                 "title" : entries[i].title,
                                                 "description" : entries[i].contentSnippet,
                                                 "link" : entries[i].link,
                                                 "isSelected" : false})
            }
        }

        onLoadResult: {
            resultsReceived = true

            if ( result.responseStatus !== 200 ) {
                // No need to show dialog in case of abort.
                if (result.responseStatus !== 0) {
                    PopupUtils.open(errorDialogComponent, appendFeedPage,
                                    {"text" : i18n.tr("Failed to perform a feed search by URL"),
                                        "title" : i18n.tr("Search failed")})
                    console.log(JSON.stringify(result))
                }
                return
            }

            clearModelDependData()

            var feed = result.responseData.feed

            searchResultsModel.append({"url" : feed.feedUrl,
                                             "title" : feed.title,
                                             "description" : feed.description,
                                             "link" : feed.link,
                                             "isSelected" : false})
        }
    }

    GoogleFeedApi {
        id: googleFeedApi
    }

    // -------------------------------------       GOOGLE API

    Column {
        id: appendFeedColumn

        anchors.top: parent.top
        anchors.topMargin: units.gu(2)
        width: parent.width
        spacing: units.gu(2)

        TextField {
            objectName: "tfFeedUrl"
            id: tfFeedUrl

            placeholderText: i18n.tr("Type a keyword or URL")

            width: parent.width - units.gu(4)
            height:units.gu(5)
            anchors {
                horizontalCenter: parent.horizontalCenter
            }
            primaryItem: Image {
                height: parent.height*0.5
                width: height
                anchors.verticalCenter: parent.verticalCenter
//                anchors.verticalCenterOffset: -units.gu(0.2)
                source: Qt.resolvedUrl("../icons_tmp/find.svg")
                smooth: true

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        if (Qt.inputMethod.visible)
                            tfFeedUrl.accapt()
                    }
                }
            }

            onAccepted: {
                accapt()
            }

            function accapt() {
                Qt.inputMethod.hide()
                var userInput = text

                if (userInput == "")
                    return

                // Very simple logic, URL if there are no spaces and contains dots.
                // But simple not means that it is wrong.
                var isUrlEntered = (userInput.indexOf(" ") === -1 && userInput.indexOf(".") !== -1)

                if (isUrlEntered) {
                    if (userInput.indexOf("http://") !== 0)
                        userInput = "http://" + userInput
                    googleFeedApi.loadFeed(userInput)
                }
                else googleFeedApi.findFeeds(text)
            }
        }

        ListItem.Header {

            ListItem.ThinDivider { }

            text: i18n.tr("Search results")
        }
    } // Column


    ListView {
        id: searchResults
        objectName: "feedpagelistview"

        width: parent.width
        clip: true
        anchors {
            bottom: parent.bottom
            // bottomMargin: appendFeedTools.height
            top: appendFeedColumn.bottom
        }

        model: searchResultsModel

        delegate: ListItem.Standard {
            text: model.title
            objectName: "feedpagelistitem"

            control: CheckBox {
                anchors.verticalCenter: parent.verticalCenter

                objectName: "feedCheckbox"
                onCheckedChanged: {
                    searchResultsModel.setProperty(index, "isSelected", checked)
                    if (checked)
                        selectedCount++
                    else selectedCount--
                }
            }
        }
    }

    ListModel {
        id: searchResultsModel
    }

    Label {
        visible: searchResultsModel.count == 0 && resultsReceived
        anchors.centerIn: parent
        text: i18n.tr("No feeds")
        fontSize: "large"
    }
}
